/*
 * Created on Nov 29, 2004
 */
package weka.classifiers.ucd.biased;

import weka.core.Instance;
import weka.core.Instances;


/**
 * @author andreas
 */
public class OverweightingBiasedClassifier extends BiasedClassifierImpl {
    
    /**
     * As there's no real max. for the overweighting bias, we have to define
     * a "reasonable" maximum, so that the bias strength can be properly
     * interpreted. 
     */
    protected static double MAX_OVERWEIGHT = 9;
    
    public OverweightingBiasedClassifier() {
        // different default bias
        bias = 4.0 / 9.0;
    }

    /* (non-Javadoc)
     * @see weka.classifiers.Classifier#buildClassifier(weka.core.Instances)
     */
    public void buildClassifier(Instances originalDataset) throws Exception {
		int nInstances = originalDataset.numInstances();
		Instances modifiedDataset = new Instances(originalDataset, 0);
		for (int m = 0; m < nInstances; m++) {
			Instance i = originalDataset.instance(m); 
			if ((originalDataset.instance(m).classValue() != classIndex)) {
				i = (Instance) i.copy();
				i.setWeight(((bias * MAX_OVERWEIGHT) + 1) * i.weight()); // incorporate previous weights!
			}
			modifiedDataset.add(i);
		}
		classifier.buildClassifier(modifiedDataset);
    }
}
