/*
 * Created on Dec 1, 2004
 */
package weka.classifiers.ucd.biased;

import weka.classifiers.evaluation.ThresholdCurve;
import weka.classifiers.meta.ThresholdSelector;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;

/**
 * Class that overrides the original Weka ThresholdSelector so that
 * Precision (instead of F1) is maximised under the condition that
 * F1 > MIN_VALUE.
 * @author andreas
 */
public class PrecisionThresholdSelector extends ThresholdSelector implements BiasedClassifier {

    protected void findThreshold(FastVector predictions) {
        
        Instances curve = (new ThresholdCurve()).getCurve(predictions, m_DesignatedClass);
        
        //System.err.println(curve);
        double low = 1.0;
        double high = 0.0;
        if (curve.numInstances() > 0) {
            // Instance maxFM = curve.instance(0); // ah
            Instance maxPrec = curve.instance(0);  // ah
            int indexFM = curve.attribute(ThresholdCurve.FMEASURE_NAME).index();
            int indexPrec = curve.attribute(ThresholdCurve.PRECISION_NAME).index(); // ah
            int indexThreshold = curve.attribute(ThresholdCurve.THRESHOLD_NAME).index();
            for (int i = 1; i < curve.numInstances(); i++) {
                Instance current = curve.instance(i);
                // if (current.value(indexFM) > maxFM.value(indexFM)) {
                if (current.value(indexPrec) > maxPrec.value(indexPrec)
                        && current.value(indexFM) > MIN_VALUE) { // ah
                    maxPrec = current;	// ah
                }
                if (m_RangeMode == RANGE_BOUNDS) {
                    double thresh = current.value(indexThreshold);
                    if (thresh < low) {
                        low = thresh;
                    }
                    if (thresh > high) {
                        high = thresh;
                    }
                }
            }
            //          if (maxFM.value(indexFM) > MIN_VALUE) {
            //            m_BestThreshold = maxFM.value(indexThreshold);
            //            m_BestValue = maxFM.value(indexFM);
            //            //System.err.println("maxFM: " + maxFM);
            //          }
            m_BestThreshold = maxPrec.value(indexThreshold);	// ah
            m_BestValue = maxPrec.value(indexPrec);				// ah
            if (m_RangeMode == RANGE_BOUNDS) {
                m_LowThreshold = low;
                m_HighThreshold = high;
                //System.err.println("Threshold range: " + low + " - " + high);
            }
        }
    }

    /* (non-Javadoc)
     * @see weka.classifiers.ucd.biased.BiasedClassifierInt#getClassIndex()
     */
    public int getPreferredClassIndex() {
        return m_DesignatedClass;
    }

    /* (non-Javadoc)
     * @see weka.classifiers.ucd.biased.BiasedClassifierInt#setClassIndex(int)
     */
    public void setPreferredClassIndex(int i) {
        if (i == 0) {
            m_ClassMode = OPTIMIZE_0;
        } else {
            m_ClassMode = OPTIMIZE_1;
        }
    }

    /* (non-Javadoc)
     * @see weka.classifiers.ucd.biased.BiasedClassifier#getBias()
     */
    public double getBiasStrength() {
        // meaningless in this case
        return 0;
    }

    /* (non-Javadoc)
     * @see weka.classifiers.ucd.biased.BiasedClassifier#setBias(double)
     */
    public void setBiasStrength(double d) {
        // meaningless in this case
        
    }

}
