/*
 * Created on 08.12.2004
 */
package weka.classifiers.ucd.biased;

import java.util.Enumeration;
import java.util.Random;

import weka.classifiers.Classifier;
import weka.classifiers.functions.SMO;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.UnsupportedClassTypeException;

/**
 * @author andreas
 */
public class RandomEnsemble extends Classifier implements BiasedClassifier {

    protected Classifier classifier = new SMO();
    
    protected int preferredClass = 0;
    
    protected int ensembleSize = 10;
    
    protected int sampleSize = 10;
    
    protected double biasStrength = 1.0;

    protected Classifier[] ensemble;
    
    protected Instances[] instances;

    /* (non-Javadoc)
	 * @see weka.classifiers.Classifier#buildClassifier(weka.core.Instances)
	 */
	public void buildClassifier(Instances trainingSet) throws Exception {
        // Check preconditions
        
        if (!trainingSet.classAttribute().isNominal()) {
            throw new UnsupportedClassTypeException("Class attribute must be nominal!");
        }

        ensemble = Classifier.makeCopies(classifier, ensembleSize); // Initialize ensemble
        
        // Sort instances
        
        int nClasses = trainingSet.numClasses();
        instances = new Instances[nClasses];
        for (int n = 0; n < nClasses; n++) {
        	instances[n] = new Instances(trainingSet, 0);
        }
        Enumeration ex = trainingSet.enumerateInstances();
        while (ex.hasMoreElements()) {
            Instance i = (Instance) ex.nextElement();
            instances[(int) i.classValue()].add(i);
        }
        
        // Randomize instances
        
        // Random rnd = new Random(seed);
        Random rnd = new Random();
        Enumeration[] en = new Enumeration[nClasses];
        for (int n = 0; n < nClasses; n++) {
        	instances[n].randomize(rnd);
        	en[n] = instances[n].enumerateInstances();
        }
        
        // Train ensemble
        
        for (int m = 0; m < ensembleSize; m++) {
        	Instances dataset = new Instances(trainingSet, 0);
        	for (int j = 0; j < sampleSize; j++) {
        		for (int n = 0; n < nClasses; n++) {
        			if (!en[n].hasMoreElements()) {
        				instances[n].randomize(rnd);
        				en[n] = instances[n].enumerateInstances();
        			}
        			if (en[n].hasMoreElements()) {
        				dataset.add((Instance) en[n].nextElement());
        			}
        		}
        	}
        	ensemble[m].buildClassifier(dataset);
        }
	}

	public double classifyInstance(Instance instance) throws Exception {
 
		if (m_Debug) System.out.print("Individual predictions:");

		int nAgree = 0;
		double altPrediction = 0;
        for (int n = 0; n < ensembleSize; n++) {
            double prediction = ensemble[n].classifyInstance(instance);
            if (m_Debug) System.out.print(" " + prediction);
            if (prediction == preferredClass) {
            	nAgree++;
            } else {
            	// Alternative prediction, if no agreement is reached
            	// In a true multiclass setup, it may be better to use
            	// the alternative prediction with the most votes instead.
            	altPrediction = prediction;
            }
        }
        double dAgree = (double) nAgree / (double) ensembleSize;
        if (m_Debug) System.out.println(" Agree: " + dAgree);
        if (dAgree < biasStrength) return altPrediction;
        return preferredClass;
		
	}
	
	/* (non-Javadoc)
	 * @see weka.classifiers.ucd.biased.BiasedClassifier#getPreferredClassIndex()
	 */
	public int getPreferredClassIndex() {
		return preferredClass;
	}

	/* (non-Javadoc)
	 * @see weka.classifiers.ucd.biased.BiasedClassifier#setPreferredClassIndex(int)
	 */
	public void setPreferredClassIndex(int i) {
		preferredClass = i;
	}
	public double getBiasStrength() {
		return biasStrength;
	}
	public void setBiasStrength(double biasStrength) {
		this.biasStrength = biasStrength;
	}
	public Classifier getClassifier() {
		return classifier;
	}
	public void setClassifier(Classifier classifier) {
		this.classifier = classifier;
	}
	public int getEnsembleSize() {
		return ensembleSize;
	}
	public void setEnsembleSize(int ensembleSize) {
		this.ensembleSize = ensembleSize;
	}
	public int getSampleSize() {
		return sampleSize;
	}
	public void setSampleSize(int sampleSize) {
		this.sampleSize = sampleSize;
	}
}
