/*
 * Created on 15.12.2004
 */
package weka.classifiers.ucd.biased;

import java.util.Enumeration;

import weka.classifiers.Classifier;
import weka.classifiers.functions.SMO;
import weka.classifiers.lazy.IB1;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.Utils;

/**
 * Classifier that uses one of two base classifiers depending on the
 * preferredClassIndex. This is a convenient way of using two different
 * classifiers on the same dataset, depending on the preferredClassIndex.
 * Note that the two classifiers are not explicitly biased.
 * @author andreas
 */
public class Splitter extends Classifier implements BiasedClassifier {
	
    protected Classifier firstClassifier = new IB1();
	protected Classifier secondClassifier = new SMO();;
	protected int preferredClassIndex = 0;

	/* (non-Javadoc)
	 * @see weka.classifiers.Classifier#buildClassifier(weka.core.Instances)
	 */
	public void buildClassifier(Instances data) throws Exception {
		if (preferredClassIndex == 0) {
			firstClassifier.buildClassifier(data);
		} else {
			secondClassifier.buildClassifier(data);
		}
	}

	/* (non-Javadoc)
	 * @see weka.classifiers.Classifier#distributionForInstance(weka.core.Instance)
	 */
	public double[] distributionForInstance(Instance instance) throws Exception {
		if (preferredClassIndex == 0) {
			return firstClassifier.distributionForInstance(instance);
		} else {
			return secondClassifier.distributionForInstance(instance);
		}
	}

	/* (non-Javadoc)
	 * @see weka.classifiers.ucd.biased.BiasedClassifier#getBiasStrength()
	 */
	public double getBiasStrength() {
		// ignore
		return 0;
	}

	/* (non-Javadoc)
	 * @see weka.classifiers.ucd.biased.BiasedClassifier#setBiasStrength(double)
	 */
	public void setBiasStrength(double d) {
		// ignore
	}

	public Classifier getFirstClassifier() {
		return firstClassifier;
	}
	
	public void setFirstClassifier(Classifier firstClassifier) {
		this.firstClassifier = firstClassifier;
	}
	
	public Classifier getSecondClassifier() {
		return secondClassifier;
	}
	
	public void setSecondClassifier(Classifier secondClassifier) {
		this.secondClassifier = secondClassifier;
	}
	
	public int getPreferredClassIndex() {
		return preferredClassIndex;
	}
	
	public void setPreferredClassIndex(int preferredClassIndex) {
		this.preferredClassIndex = preferredClassIndex;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if (preferredClassIndex == 0) {
			return firstClassifier.toString();
		} else {
			return secondClassifier.toString();
		}
	}
	
    public String[] getOptions() {
        String[] options = new String[4];
        int c = 0;
        options[c++] = "-A";
        options[c++] = firstClassifier.getClass().getName() + " " +
        	Utils.joinOptions(firstClassifier.getOptions());
        options[c++] = "-B";
        options[c++] = secondClassifier.getClass().getName() + " " +
        	Utils.joinOptions(secondClassifier.getOptions());
        return options;
    }
    
    public Enumeration listOptions() {
        // TODO Auto-generated method stub
        return super.listOptions();
    }
    
    public void setOptions(String[] options) throws Exception {
        String firstString = Utils.getOption('A', options);
        String[] firstSpec = Utils.splitOptions(firstString);
        String firstName;
        if (firstSpec.length > 0) {
            firstName = firstSpec[0];
            firstSpec[0] = "";
            firstClassifier = Classifier.forName(firstName, firstSpec);
        }
        String secondString = Utils.getOption('B', options);
        String[] secondSpec = Utils.splitOptions(secondString);
        String secondName;
        if (secondSpec.length > 0) {
            secondName = secondSpec[0];
            secondSpec[0] = "";
            secondClassifier = Classifier.forName(secondName, secondSpec);
        }
    }

}
