/*
 * Created on 05.12.2004
 */
package weka.classifiers.ucd.ensemble;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.*;

import weka.classifiers.Classifier;
import weka.classifiers.functions.SMO;
import weka.classifiers.ucd.eval.AbstractEvaluatingClassifier;
import weka.classifiers.ucd.eval.RelabelUtils;
import weka.core.*;

/**
 * @author andreas
 */
public class DivideAndConquer extends AbstractEvaluatingClassifier {
	
	public DivideAndConquer() {
        // different default classifier than the abstract super class
        classifier = new SMO();
	}
	
	// protected int rounds = 2;
	
	protected int nClasses;

	/* (non-Javadoc)
	 * @see weka.classifiers.Classifier#buildClassifier(weka.core.Instances)
	 */
	public void buildClassifier(Instances instances) throws Exception {
        if (!instances.classAttribute().isNominal()) {
            throw new UnsupportedClassTypeException("Class attribute must be nominal!");
        }
        
        AttributeStats stats = instances.attributeStats(instances.classIndex());

        // If data contains only one instance of positive data
        // optimize on training data

        if (stats.distinctCount < 2) {
            System.err.println("Couldn't find examples of both classes. No adjustment.");
            classifier.buildClassifier(instances);
            return;
        }
       
        // Initialize everything
    
        nClasses = instances.numClasses();
        rnd = new Random(seed);
        
        for (int n = 0; n < maxRounds; n++) {
        	initInstanceSet(instances);
        	RelabelUtils ru = new RelabelUtils(relabeledInstances, relabelMatrix);
        	FastVector predictions = getPredictions(instances, ru);
        	instances = relabeledInstances;
			BufferedWriter bw = new BufferedWriter(new FileWriter("round" + n + "relabeled.arff"));
			bw.write(instances.toString());
			bw.flush();
			bw.close();
       }
        
        classifier.buildClassifier(instances);
	}
	
	public double[] distributionForInstance(Instance i) throws Exception {
		Instance j = (Instance) i.copy();
		j.setDataset(relabeledInstances);
		double[] predictions = classifier.distributionForInstance(j);
		double[] newPredictions = new double[nClasses];
		for (int n = 0; n < newPredictions.length; n++) {
			newPredictions[n] = 0;
		}
		for (int n = 0; n < predictions.length; n++) {
			Integer actualClassI = (Integer) labelMap.get(new Integer(n));
			int actualClass = n;
			if (actualClassI != null) actualClass = actualClassI.intValue();
			newPredictions[actualClass] += predictions[n];
		}
		return newPredictions;
	}

	protected Map labelMap = new HashMap();
	// protected Instances relabeledInstances;
	protected double[][] relabelMatrix;
	
	/**
	 * Creates a new, empty instance set with twice as many class labels as
	 * the original instance set. Keeps track of mappings from the new labels
	 * to the original ones.
	 * @param old
	 * @param prefix
	 * @return
	 */
	protected void initInstanceSet(Instances old) {
		int nco = old.numClasses();
		FastVector attInfo = new FastVector();
		for (int k = 0; k < old.numAttributes(); k++) {
			if (k == old.classIndex()) {
				FastVector attValues = new FastVector();
				for (int n = 0; n < nco * 2; n++) {
					attValues.addElement("class" + n);
				}
				attInfo.addElement(new Attribute("class", attValues));
			} else {
				attInfo.addElement(old.attribute(k).copy());
			}
		}
		relabeledInstances = new Instances("relabeledInstances", attInfo, nco);
		relabeledInstances.setClassIndex(old.classIndex());
		relabelMatrix = new double[nco][nco];
		for (int n = 0; n < nco; n++) {
			for (int m = 0; m < nco; m++) {
				if (n == m) {
					relabelMatrix[n][m] = n;
				} else {
					relabelMatrix[n][m] = nco + m;
					Integer key = new Integer(n);
					Integer mm = new Integer(m);
					if (labelMap.containsKey(mm)) {
						labelMap.put(new Integer(nco + m), labelMap.get(mm));
					} else {
						labelMap.put(new Integer(nco + m), mm);
					}
				}
			}
		}
	}
	
	public String toString() {
		return classifier.toString();
	}

    /**
     * @return Returns the classifier.
     */
    public Classifier getClassifier() {
        return classifier;
    }

    /**
     * @param classifier The classifier to set.
     */
    public void setClassifier(Classifier classifier) {
        this.classifier = classifier;
    }
}
