/*
 * Created on Dec 9, 2004
 */
package weka.classifiers.ucd.ensemble;

import java.util.Enumeration;
import java.util.Random;

import weka.classifiers.Classifier;
import weka.classifiers.functions.SMO;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.UnsupportedClassTypeException;

/**
 * @author andreas
 */
public class RandomClassifier extends Classifier {
    
    protected Classifier classifier = new SMO();
    protected int sampleSize = 20;

    /* (non-Javadoc)
     * @see weka.classifiers.Classifier#buildClassifier(weka.core.Instances)
     */
    public void buildClassifier(Instances trainingSet) throws Exception {

        // Check preconditions
        
        if (!trainingSet.classAttribute().isNominal()) {
            throw new UnsupportedClassTypeException("Class attribute must be nominal!");
        }
        
        // Sort instances
        
        int nClasses = trainingSet.numClasses();
        Instances[] instances = new Instances[nClasses];
        for (int n = 0; n < nClasses; n++) {
        	instances[n] = new Instances(trainingSet, 0);
        }
        Enumeration ex = trainingSet.enumerateInstances();
        while (ex.hasMoreElements()) {
            Instance i = (Instance) ex.nextElement();
            instances[(int) i.classValue()].add(i);
        }
        
        // Randomize instances
        
        // Random rnd = new Random(seed);
        Random rnd = new Random();
        Enumeration[] en = new Enumeration[nClasses];
        for (int n = 0; n < nClasses; n++) {
        	instances[n].randomize(rnd);
        	en[n] = instances[n].enumerateInstances();
        }
        
        // Select instances
        
    	Instances dataset = new Instances(trainingSet, 0);
    	for (int j = 0; j < sampleSize; j++) {
    		for (int n = 0; n < nClasses; n++) {
    			if (!en[n].hasMoreElements()) {
    				instances[n].randomize(rnd);
    				en[n] = instances[n].enumerateInstances();
    			}
    			if (en[n].hasMoreElements()) {
    				dataset.add((Instance) en[n].nextElement());
    			}
    		}
    	}
    	
    	// Train
    	
    	classifier.buildClassifier(dataset);
 
    }
    
    

    public double[] distributionForInstance(Instance instance) throws Exception {
        return classifier.distributionForInstance(instance);
    }
    
    public Classifier getClassifier() {
        return classifier;
    }
    
    public void setClassifier(Classifier classifier) {
        this.classifier = classifier;
    }
    
    public int getSampleSize() {
        return sampleSize;
    }
    
    public void setSampleSize(int sampleSize) {
        this.sampleSize = sampleSize;
    }
}
